import { world, system, GameMode } from "@minecraft/server";

world.beforeEvents.worldInitialize.subscribe(initEvent => {
    initEvent.blockComponentRegistry.registerCustomComponent("betterbiomes:blueSapling", {
        onPlayerInteract: eventData => {
            const player = eventData.player;
            const block = eventData.block;
            const blockLocation = block.location;
            const playerEquippable = player.getComponent("equippable");
            const itemUsed = playerEquippable.getEquipment("Mainhand");

            function decreaseItem() {
                if (!player.matches({ gameMode: GameMode.creative })) {
                    if (itemUsed.amount > 1) {
                        itemUsed.amount--;
                        playerEquippable.setEquipment("Mainhand", itemUsed);
                        console.log(`Item decremented. Remaining amount: ${itemUsed.amount}`);
                    } else {
                        playerEquippable.setEquipment("Mainhand", undefined);
                        console.log("Item used up. Removed from main hand.");
                    }
                }
            }

            if (itemUsed === undefined) {
                console.log("No item used in main hand.");
                return;
            }

            // Verifica que el bloque sea el correcto y que el objeto usado sea "bone_meal"
            if (block.typeId === "betterbiomes:blue_sapling" && itemUsed.typeId === "minecraft:bone_meal") {
                console.log("Bone meal used on sapling block at", blockLocation);
                system.run(() => {
                    decreaseItem();

                    block.dimension.playSound("item.bone_meal.use", blockLocation, { pitch: 1.0, volume: 1.0 });
                    block.dimension.spawnParticle("minecraft:crop_growth_emitter", { x: blockLocation.x + 0.5, y: blockLocation.y + 0.5, z: blockLocation.z + 0.5 });

                    // Altura del tronco ajustada a 3 (1 visible)
                    const trunkHeight = 3; // Altura del tronco
                    console.log(`Generated tree trunk height: ${trunkHeight}`);

                    // Comando para construir el árbol
                    const commands = [];

                    // Genera el tronco
                    for (let i = 0; i < trunkHeight; i++) {
                        const trunkLocation = `${blockLocation.x} ${blockLocation.y + i} ${blockLocation.z}`;
                        commands.push(`setblock ${trunkLocation} betterbiomes:blue_log`);
                        console.log(`Preparing to place trunk block at: ${trunkLocation}`);
                    }

                    // Generar copa frondosa en forma de pino
                    const leafOffsets = [];
                    const leafHeight = 7; // Altura total de la copa
                    const baseLeafRadius = 6; // Radio de la base de la copa, más amplio

                    for (let y = 0; y < leafHeight; y++) {
                        // El radio decrece a medida que aumenta la altura
                        const currentLeafRadius = baseLeafRadius - y; // Disminuye el radio para dar forma de pino
                        for (let x = -currentLeafRadius; x <= currentLeafRadius; x++) {
                            for (let z = -currentLeafRadius; z <= currentLeafRadius; z++) {
                                const distanceFromCenter = Math.abs(x) + Math.abs(z);
                                if (distanceFromCenter <= currentLeafRadius) {
                                    // Añade los offsets para las hojas en la copa
                                    leafOffsets.push({ x, y: trunkHeight + y, z }); // Ajusta la altura de las hojas
                                }
                            }
                        }
                    }

                    // Colocar hojas con forma de pino
                    leafOffsets.forEach(offset => {
                        const leafLocation = `${blockLocation.x + offset.x} ${blockLocation.y + offset.y} ${blockLocation.z + offset.z}`;
                        commands.push(`setblock ${leafLocation} betterbiomes:blue_leaves`);
                        console.log(`Preparing to place leaf block at: ${leafLocation}`);
                    });

                    // Ejecuta todos los comandos
                    commands.forEach(cmd => {
                        block.dimension.runCommand(cmd);
                    });
                });
            } else {
                console.log("Incorrect block type or item used.");
                return;
            }
        }
    });
});