import { world, system, GameMode } from "@minecraft/server";

world.beforeEvents.worldInitialize.subscribe(initEvent => {
  initEvent.blockComponentRegistry.registerCustomComponent("betterbiomes:brownSapling", {
    onPlayerInteract: eventData => {
      const player = eventData.player;
      const block = eventData.block;
      const blockLocation = block.location;
      const playerEquippable = player.getComponent("equippable");
      const itemUsed = playerEquippable.getEquipment("Mainhand");

      function decreaseItem() {
        if (!player.matches({ gameMode: GameMode.creative })) {
          if (itemUsed.amount > 1) {
            itemUsed.amount--;
            playerEquippable.setEquipment("Mainhand", itemUsed);
            console.log(`Item decremented. Remaining amount: ${itemUsed.amount}`);
          } else {
            playerEquippable.setEquipment("Mainhand", undefined);
            console.log("Item used up. Removed from main hand.");
          }
        }
      }

      if (itemUsed === undefined) {
        console.log("No item used in main hand.");
        return;
      }

      // Verifica que el bloque sea el correcto y que el objeto usado sea "bone_meal"
      if (block.typeId === "betterbiomes:brown_sapling" && itemUsed.typeId === "minecraft:bone_meal") {
        console.log("Bone meal used on sapling block at", blockLocation);
        system.run(() => {
          decreaseItem();

          block.dimension.playSound("item.bone_meal.use", blockLocation, { pitch: 1.0, volume: 1.0 });
          block.dimension.spawnParticle("minecraft:crop_growth_emitter", { x: blockLocation.x + 0.5, y: blockLocation.y + 0.5, z: blockLocation.z + 0.5 });

          // Altura del tronco entre 5 y 8
          const trunkHeight = Math.floor(Math.random() * 4) + 5; // Genera un número entre 5 y 8
          console.log(`Generated tree trunk height: ${trunkHeight}`);

          const commands = [];

          // Genera el tronco
          for (let i = 0; i < trunkHeight; i++) {
            const trunkLocation = `${blockLocation.x} ${blockLocation.y + i} ${blockLocation.z}`;
            commands.push(`setblock ${trunkLocation} betterbiomes:brown_log`);
            console.log(`Preparing to place trunk block at: ${trunkLocation}`);
          }

          // Generar la copa con desplazamiento
          const leafOffsets = [];
          const canopyHeight = 3; // Altura de la copa
          const baseLeafRadius = 3; // Radio base de la copa

          // Generar hojas en forma de copa
          for (let y = 0; y <= canopyHeight; y++) {
            const currentLeafRadius = baseLeafRadius - Math.floor(y / 2); // Reducir el radio conforme aumenta la altura

            for (let x = -currentLeafRadius; x <= currentLeafRadius; x++) {
              for (let z = -currentLeafRadius; z <= currentLeafRadius; z++) {
                const distanceFromCenter = Math.abs(x) + Math.abs(z);
                if (distanceFromCenter <= currentLeafRadius) {
                  // Agujeros en la parte superior de la copa
                  if (y === canopyHeight && Math.random() < 0.25) continue; // Agujeros aleatorios
                  leafOffsets.push({ x, y: trunkHeight + y, z }); // Ajusta la altura de las hojas
                }
              }
            }
          }

          leafOffsets.forEach(offset => {
            const leafLocation = `${blockLocation.x + offset.x} ${blockLocation.y + offset.y} ${blockLocation.z + offset.z}`;
            commands.push(`setblock ${leafLocation} betterbiomes:brown_leaves`);
            console.log(`Preparing to place leaf block at: ${leafLocation}`);
          });

          // Ejecuta todos los comandos
          commands.forEach(cmd => {
            block.dimension.runCommand(cmd);
          });
        });
      } else {
        console.log("Incorrect block type or item used.");
        return;
      }
    }
  });
});