import { world, system, GameMode } from "@minecraft/server";

world.beforeEvents.worldInitialize.subscribe(initEvent => {
    initEvent.blockComponentRegistry.registerCustomComponent("betterbiomes:desertSapling", {
        onPlayerInteract: eventData => {
            const player = eventData.player;
            const block = eventData.block;
            const blockLocation = block.location;
            const playerEquippable = player.getComponent("equippable");
            const itemUsed = playerEquippable.getEquipment("Mainhand");

            function decreaseItem() {
                if (!player.matches({ gameMode: GameMode.creative })) {
                    if (itemUsed.amount > 1) {
                        itemUsed.amount--;
                        playerEquippable.setEquipment("Mainhand", itemUsed);
                        console.log(`Item decremented. Remaining amount: ${itemUsed.amount}`);
                    } else {
                        playerEquippable.setEquipment("Mainhand", undefined);
                        console.log("Item used up. Removed from main hand.");
                    }
                }
            }

            if (itemUsed === undefined) {
                console.log("No item used in main hand.");
                return;
            }

            // Verifica que el bloque sea el correcto y que el objeto usado sea "bone_meal"
            if (block.typeId === "betterbiomes:desert_sapling" && itemUsed.typeId === "minecraft:bone_meal") {
                console.log("Bone meal used on sapling block at", blockLocation);
                system.run(() => {
                    decreaseItem();

                    block.dimension.playSound("item.bone_meal.use", blockLocation, { pitch: 1.0, volume: 1.0 });
                    block.dimension.spawnParticle("minecraft:crop_growth_emitter", { x: blockLocation.x + 0.5, y: blockLocation.y + 0.5, z: blockLocation.z + 0.5 });

                    // Crear tronco recto y luego curvo
                    const trunkHeight = 4; // Altura recta del tronco
                    const curveHeight = Math.floor(Math.random() * 2) + 3; // Altura curva adicional
                    const trunkCurveDirection = Math.random() < 0.5 ? -1 : 1; // Dirección de la curva (izquierda o derecha)
                    let currentX = blockLocation.x;

                    const commands = [];

                    // Genera el tronco recto
                    for (let i = 0; i < trunkHeight; i++) {
                        const trunkLocation = `${blockLocation.x} ${blockLocation.y + i} ${blockLocation.z}`;
                        commands.push(`setblock ${trunkLocation} minecraft:acacia_log`);
                        console.log(`Preparing to place trunk block at: ${trunkLocation}`);
                    }

                    // Genera el tronco curvo
                    for (let i = 0; i < curveHeight; i++) {
                        currentX += trunkCurveDirection; // Inclina el tronco hacia un lado
                        const trunkLocation = `${currentX} ${blockLocation.y + trunkHeight + i} ${blockLocation.z}`;
                        commands.push(`setblock ${trunkLocation} minecraft:acacia_log`);
                        console.log(`Preparing to place curved trunk block at: ${trunkLocation}`);
                    }

                    // Generar la copa extendida
                    const canopyStart = trunkHeight + curveHeight; // Comienza la copa desde el final del tronco
                    const canopyRadius = 7; // Radio ampliado de la copa
                    const leafOffsets = [];

                    // Crear una copa extendida pero dispersa, con más hojas colgando en los bordes
                    for (let y = 0; y <= 3; y++) { // Copa con altura de 4 bloques
                        const currentLeafRadius = canopyRadius - y; // Disminuye el radio en cada nivel
                        for (let x = -currentLeafRadius; x <= currentLeafRadius; x++) {
                            for (let z = -currentLeafRadius; z <= currentLeafRadius; z++) {
                                const distanceFromCenter = Math.abs(x) + Math.abs(z);
                                if (distanceFromCenter <= currentLeafRadius) {
                                    leafOffsets.push({ x: currentX + x, y: blockLocation.y + canopyStart + y, z: blockLocation.z + z });
                                }
                            }
                        }
                    }

                    // Colocar las hojas con cascada sin enredaderas
                    leafOffsets.forEach(offset => {
                        const leafLocation = `${offset.x} ${offset.y} ${offset.z}`;
                        commands.push(`setblock ${leafLocation} betterbiomes:desert_leaves`);
                        console.log(`Preparing to place leaf block at: ${leafLocation}`);
                        
                        // Añadir bloques de hojas en cascada debajo de los bordes
                        if (Math.random() < 0.7 && offset.y === blockLocation.y + canopyStart) { // Solo en el borde superior
                            const cascadeDepth = Math.floor(Math.random() * 2) + 2; // Profundidad aleatoria de la cascada
                            for (let hang = 1; hang <= cascadeDepth; hang++) {
                                const cascadeLeafLocation = `${offset.x} ${offset.y - hang} ${offset.z}`;
                                commands.push(`setblock ${cascadeLeafLocation} betterbiomes:desert_leaves`);
                                console.log(`Preparing to place cascading leaf block at: ${cascadeLeafLocation}`);
                            }
                        }
                    });

                    // Ejecuta todos los comandos
                    commands.forEach(cmd => {
                        block.dimension.runCommand(cmd);
                    });
                });
            } else {
                console.log("Incorrect block type or item used.");
                return;
            }
        }
    });
});