import { world, BlockPermutation, ItemStack, EquipmentSlot } from "@minecraft/server";

world.beforeEvents.worldInitialize.subscribe(initEvent => {
    let tickCount = 0;
    const validLogBlocks = new Set(["betterbiomes:lila_log"]);
    const radius = 8;

    initEvent.blockComponentRegistry.registerCustomComponent('betterbiomes:falling_leaves', {
        beforeOnPlayerPlace: e => {
            e.permutationToPlace = e.permutationToPlace.withState('betterbiomes:leavesPlaced', true);
        },
        onRandomTick: e => {
            tickCount++;
            if (tickCount < 3) return;
            tickCount = 0;

            const loc = e.block.location;
            let foundLog = false;

            // Búsqueda de troncos cercanos
            outer: for (let xOffset = -radius; xOffset <= radius; xOffset++) {
                for (let yOffset = -radius; yOffset <= radius; yOffset++) {
                    for (let zOffset = -radius; zOffset <= radius; zOffset++) {
                        const blockLoc = {
                            x: loc.x + xOffset,
                            y: loc.y + yOffset,
                            z: loc.z + zOffset,
                        };
                        const block = e.block.dimension.getBlock(blockLoc);

                        if (validLogBlocks.has(block?.typeId)) {
                            foundLog = true;
                            break outer;
                        }
                    }
                }
            }

            // Si no hay tronco, la hoja desaparece y genera loot
            if (!foundLog) {
                if (e.block.permutation.getState('betterbiomes:leavesPlaced') === false) {
                    e.block.setType('air');
                    dropLilaLeavesLoot(e.block);
                }
            }
        },

        onPlayerDestroy: e => {
            const player = e.player;
            const block = e.block;
            const item = player.getComponent("minecraft:equippable")?.getEquipment(EquipmentSlot.Mainhand);

            if (item?.typeId === "minecraft:shears") {
                // Si el jugador usa tijeras, suelta el bloque de hojas en sí
                block.dimension.spawnItem(new ItemStack("betterbiomes:lila_leaves", 1), block.location);
            } else {
                // Si rompe con la mano, hay una pequeña probabilidad de obtener una manzana
                dropLilaLeavesLoot(block, item === undefined);
            }
        }
    });
});

// Función para manejar la generación de loot
function dropLilaLeavesLoot(block, isHand) {
    const dimension = block.dimension;
    const loc = block.location;

    // Probabilidad fija de dropear un retoño lila (5%)
    if (Math.random() < 0.05) {
        dimension.spawnItem(new ItemStack("betterbiomes:lila_sapling", 1), loc);
    }
    
    // Probabilidad fija de dropear un palo (2%)
    if (Math.random() < 0.02) {
        dimension.spawnItem(new ItemStack("minecraft:stick", 1), loc);
    }

    // Si el jugador rompió con la mano, hay 1% de probabilidad de obtener una manzana
    if (isHand && Math.random() < 0.01) {
        dimension.spawnItem(new ItemStack("minecraft:apple", 1), loc);
    }
}