import { world, system, GameMode } from "@minecraft/server";

world.beforeEvents.worldInitialize.subscribe(initEvent => {
    initEvent.blockComponentRegistry.registerCustomComponent("betterbiomes:greenSapling", {
        onPlayerInteract: eventData => {
            const player = eventData.player;
            const block = eventData.block;
            const blockLocation = block.location;
            const playerEquippable = player.getComponent("equippable");
            const itemUsed = playerEquippable.getEquipment("Mainhand");

            function decreaseItem() {
                if (!player.matches({ gameMode: GameMode.creative })) {
                    if (itemUsed.amount > 1) {
                        itemUsed.amount--;
                        playerEquippable.setEquipment("Mainhand", itemUsed);
                        console.log(`Item decremented. Remaining amount: ${itemUsed.amount}`);
                    } else {
                        playerEquippable.setEquipment("Mainhand", undefined);
                        console.log("Item used up. Removed from main hand.");
                    }
                }
            }

            function getRandomLeafType() {
                return Math.random() < 0.5 ? "betterbiomes:green_leaves" : "betterbiomes:green_leaves1";
            }

            if (itemUsed === undefined) {
                console.log("No item used in main hand.");
                return;
            }

            if (block.typeId === "betterbiomes:green_sapling" && itemUsed.typeId === "minecraft:bone_meal") {
                console.log("Bone meal used on sapling block at", blockLocation);
                system.run(() => {
                    decreaseItem();

                    block.dimension.playSound("item.bone_meal.use", blockLocation, { pitch: 1.0, volume: 1.0 });
                    block.dimension.spawnParticle("minecraft:crop_growth_emitter", { x: blockLocation.x + 0.5, y: blockLocation.y + 0.5, z: blockLocation.z + 0.5 });

                    const trunkHeight = 6;
                    const commands = [];

                    // Generar tronco
                    for (let i = 0; i < trunkHeight; i++) {
                        const trunkLocation = `${blockLocation.x} ${blockLocation.y + i} ${blockLocation.z}`;
                        commands.push(`setblock ${trunkLocation} betterbiomes:green_log`);
                    }

                    const canopyStart = trunkHeight;
                    const canopyRadius = 4;
                    const leafOffsets = [];

                    // Crear una copa más redonda
                    for (let y = -2; y <= 2; y++) {
                        for (let x = -canopyRadius; x <= canopyRadius; x++) {
                            for (let z = -canopyRadius; z <= canopyRadius; z++) {
                                const distance = Math.sqrt(x * x + y * y * 1.2 + z * z);
                                if (distance <= canopyRadius) {
                                    leafOffsets.push({ x: blockLocation.x + x, y: blockLocation.y + canopyStart + y, z: blockLocation.z + z });
                                }
                            }
                        }
                    }

                    // Colocar hojas y ramas colgantes
                    leafOffsets.forEach(offset => {
                        const leafType = getRandomLeafType();
                        const leafLocation = `${offset.x} ${offset.y} ${offset.z}`;
                        commands.push(`setblock ${leafLocation} ${leafType}`);

                        // Generar hojas colgantes en los bordes inferiores
                        if (Math.random() < 0.6) { 
                            const hangLength = Math.floor(Math.random() * 3) + 1; 
                            for (let i = 1; i <= hangLength; i++) {
                                const hangingLeafType = getRandomLeafType();
                                const hangingLeafLocation = `${offset.x} ${offset.y - i} ${offset.z}`;
                                commands.push(`setblock ${hangingLeafLocation} ${hangingLeafType}`);
                            }
                        }
                    });

                    // Ejecutar los comandos
                    commands.forEach(cmd => {
                        block.dimension.runCommand(cmd);
                    });
                });
            } else {
                console.log("Incorrect block type or item used.");
                return;
            }
        }
    });
});