import { EquipmentSlot, GameMode, world, ItemStack } from "@minecraft/server";

/**
 * @param {number} min The minimum integer
 * @param {number} max The maximum integer
 * @returns {number} A random integer between the `min` and `max` parameters (inclusive)
 */
const randomInt = (min, max) => Math.floor(Math.random() * (max - min + 1)) + min;

const maxGrowth = 3; // Máximo de estados de crecimiento

/** @type {import("@minecraft/server").BlockCustomComponent} */
const CustomFlowerGrowthComponent = {
    onPlayerInteract({ block, dimension, player }) {
        const equippable = player.getComponent("minecraft:equippable");
        if (!equippable) return;

        const mainhand = equippable.getEquipmentSlot(EquipmentSlot.Mainhand);
        
        // Comprobar si se utiliza "bone_meal" o "lila_flowers_item"
        if (!mainhand.hasItem() || (mainhand.typeId !== "minecraft:bone_meal" && mainhand.typeId !== "betterbiomes:lila_flowers_item")) return;

        let growth = block.permutation.getState("betterbiomes:growth");

        // Incrementar el estado de crecimiento hasta el máximo de 3
        if (growth < maxGrowth) {
            growth += 1;
            block.setPermutation(block.permutation.withState("betterbiomes:growth", growth)); // Asegúrate de que esto es correcto

            // Hacer aparecer una nueva flor en la ubicación del bloque
            dimension.spawnItem(new ItemStack("betterbiomes:lila_flowers_item", 1), block.location.add(0, 0, 0));

            // Jugar efectos
            const effectLocation = block.center();
            dimension.playSound("item.bone_meal.use", effectLocation);
            dimension.spawnParticle("minecraft:crop_growth_emitter", effectLocation);
        }

        // Si se alcanza el estado máximo (3), no hacer nada
        if (growth >= maxGrowth) {
            // Ya no se envía un mensaje al jugador
        }
    },
};

world.beforeEvents.worldInitialize.subscribe(({ blockComponentRegistry }) => {
    blockComponentRegistry.registerCustomComponent("betterbiomes:onInteract", CustomFlowerGrowthComponent);
});