import { world, system, BlockPermutation, ItemStack, GameMode } from "@minecraft/server";

world.beforeEvents.itemUseOn.subscribe(eventData => {
  const player = eventData.source;
  const itemUsed = eventData.itemStack;
  const block = eventData.block;
  const blockLocation = block.location;
  const playerEquippable = player.getComponent("equippable");
  const stateAll = block.permutation.getAllStates();

  function damageDurability() {
    if (!player.matches({ gameMode: GameMode.creative })) {
      const itemEnchantable = itemUsed.getComponent("minecraft:enchantable");
      const unbreakingLevel = itemEnchantable?.getEnchantment("unbreaking")?.level ?? 0;
      const breakChance = 100 / (unbreakingLevel + 1);
      const randomizeChance = Math.random() * 100;
      if (breakChance < randomizeChance) return;
      const itemDurability = itemUsed.getComponent("durability");
      if (!itemDurability) return;
      const itemMaxDurability = itemDurability.maxDurability;
      const itemDurabilityDamage = itemDurability.damage;
      const durabilityCalculation = itemMaxDurability - itemDurabilityDamage;
      let damageAmount = 1;
      const currentDurability = durabilityCalculation - damageAmount;
      if (currentDurability <= 0) {
        player.playSound("random.break", { pitch: 0.9, location: player.location, volume: 1 });
        playerEquippable.setEquipment("Mainhand", null);
      } else {
        itemDurability.damage += damageAmount;
        playerEquippable.setEquipment("Mainhand", itemUsed);
      }
    }
  }

  if (itemUsed === undefined) return;

  // Comprobamos si el jugador usa un hacha
  if (itemUsed.hasTag("minecraft:is_axe")) {
    // Definimos un objeto para mapear los troncos a sus versiones peladas
    const logToStripped = {
      "betterbiomes:blue_log": "betterbiomes:blue_stripped_log",
      "betterbiomes:brown_log": "betterbiomes:brown_stripped_log",
      "betterbiomes:green_log": "betterbiomes:green_stripped_log",
      "betterbiomes:lila_log": "betterbiomes:lila_stripped_log",
      "betterbiomes:pink_log": "betterbiomes:pink_stripped_log"
    };

    // Verificamos si el bloque es uno de los troncos definidos
    if (logToStripped[block.typeId]) {
      system.run(() => {
        damageDurability();  // Aplica daño a la durabilidad de la herramienta
        block.dimension.playSound("use.wood", blockLocation, { pitch: 0.8, volume: 1.0 });
        block.setPermutation(BlockPermutation.resolve(logToStripped[block.typeId], stateAll));  // Cambia el bloque a su versión pelada
      });
    }
  }
});